@@ -1,7 +1,7 @@
 from urllib.parse import quote
 from os.path import join
 
-__version__ = "2.0.0.47"
+__version__ = "2.0.0.50"
 
 def _pma_join(*s):
 	joinstring = ""
@@ -9,3 +9,10 @@ def _pma_join(*s):
 		if not (ss is None):
 			joinstring = join(joinstring, ss)
 	return joinstring.replace("\\", "/")
+
+def _pma_q(arg):
+	if (arg is None):
+		return ''
+	else:
+		return quote(str(arg), safe='')
+	
